//
//  DPCurrencyHandler.m
//  Currency
//
//  Created by Hjalti Jakobsson on 6.8.2010.
//  Copyright 2010 Hjalti Jakobsson. All rights reserved.
//

#import "DPCurrencyHandler.h"
#import "Constants.h"
#import "TouchXML.h"
#import "DPCurrency.h"

#define kCurrencyElement @"GjaldmidillRow"
#define kCurrencyIdentifier @"Mynt"
#define kCurrencyName @"Heiti"
#define kCurrencyBuy @"Kaup"
#define kCurrencySell @"Sala"
#define kCurrencyIconURL @"IconUrl"

@interface DPCurrencyHandler()
- (NSArray*)parseData:(NSString*)data;
@end

@implementation DPCurrencyHandler
@synthesize delegate, isRefreshing;

- (void)update
{	
	isRefreshing = YES;
	
	NSString *urlString = [NSString stringWithFormat:CURRENCY_XML_URL];
	
	NSURL *url = [NSURL URLWithString:urlString];
	NSMutableURLRequest *urlRequest = [NSMutableURLRequest requestWithURL:url];
	
	NSURLConnection *theConnection = [[NSURLConnection alloc] initWithRequest:urlRequest delegate:self];
	
	if(theConnection)
	{
		receivedData = [[NSMutableData data] retain];
	}
	else
	{
		isRefreshing = NO;
		
		if([delegate respondsToSelector:@selector(updateFailedWithError:)])
			[delegate performSelector:@selector(updateFailedWithError:) withObject:nil];
	}
}

- (NSArray*)parseData:(NSString*)data
{
	NSError *error = nil;
	
    CXMLDocument *xml = [[CXMLDocument alloc] initWithXMLString:data options:CXMLDocumentTidyXML error:&error];
	
	CXMLElement *root = [xml rootElement];
	
	NSMutableArray *allCurrencies = [NSMutableArray array];
	
	for(CXMLElement *element in [root children])
	{
		if([[element name] isEqualToString:kCurrencyElement])
		{
			DPCurrency *currency = [[DPCurrency alloc] init];
			
			for(CXMLElement *child in [element children])
			{		
				if([[child name] isEqualToString:kCurrencyIdentifier])
					currency.identifier = [child stringValue];
				
				if([[child name] isEqualToString:kCurrencyName])
					currency.name = [child stringValue];
				
				if([[child name] isEqualToString:kCurrencyBuy])
					currency.buy = [[child stringValue] floatValue];
				
				if([[child name] isEqualToString:kCurrencySell])
					currency.sell = [[child stringValue] floatValue];
				
				if([[child name] isEqualToString:kCurrencyIconURL])
					currency.iconURL = [NSURL URLWithString:[child stringValue]];
			}
			
			[allCurrencies addObject:currency];
			[currency release];
		}
	}
	
	[xml release];
	
	return allCurrencies;
}

- (void)handleSearchCompleted:(NSData*)data
{
	NSString *stringData = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
	NSArray *items = [self parseData:stringData];
	[stringData release];
	
	if([delegate respondsToSelector:@selector(updateCompletedWithResults:)])
		[delegate performSelector:@selector(updateCompletedWithResults:) withObject:items];
}

#pragma mark -
#pragma mark NSURLConnection delegates

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    [receivedData setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    [receivedData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{	
    [connection release];
    [receivedData release];
	
	isRefreshing = NO;
	
	if([delegate respondsToSelector:@selector(updateFailedWithError:)])
		[delegate performSelector:@selector(updateFailedWithError:) withObject:error];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{	
	[self handleSearchCompleted:receivedData];
	
    [connection release];
    [receivedData release];
	
	isRefreshing = NO;
}


@end
