//
//  DetailViewController.m
//  Currency
//
//  Created by Hjalti Jakobsson on 8.8.2010.
//  Copyright 2010 Hjalti Jakobsson. All rights reserved.
//

#import "DetailViewController.h"
#import "Currency.h"
#import "Record.h"
#import "PlotViewController.h"

@implementation DetailViewController
@synthesize selectedCurrency;

#pragma mark -
#pragma mark View lifecycle


- (void)viewDidLoad {
    [super viewDidLoad];
	
	dateFormat = [[NSDateFormatter alloc] init];
	[dateFormat setDateFormat:@"dd.MM.yyyy"];
	
	self.title = selectedCurrency.identifier;
	
	isShowingLandscapeView = NO;
	
	plotViewController = [[PlotViewController alloc] initWithNibName:@"PlotViewController" bundle:nil];
	
	[[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(orientationChanged:)
												 name:UIDeviceOrientationDidChangeNotification
											   object:nil];
}

- (void)orientationChanged:(NSNotification *)notification
{
    UIDeviceOrientation deviceOrientation = [UIDevice currentDevice].orientation;
	
    if (UIDeviceOrientationIsLandscape(deviceOrientation) && !isShowingLandscapeView)
    {
		isShowingLandscapeView = YES;
		plotViewController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
		plotViewController.currentCurrency = selectedCurrency;
        [self presentModalViewController:plotViewController animated:YES];

    }
    else if (deviceOrientation == UIDeviceOrientationPortrait && isShowingLandscapeView)
    {
        [self dismissModalViewControllerAnimated:YES];
        isShowingLandscapeView = NO;
    }
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
	if(selectedCurrency != nil)
	{
		return 3;
	}
	
	return 0;
}

- (NSString*)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	if(section == 0)
	{
		return selectedCurrency.name;
	}
	
	return nil;
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier] autorelease];
    }
	
	Record *latestRecord = [selectedCurrency latestRecord];
	
	switch(indexPath.row)
	{
		case 0:
			cell.textLabel.text = @"Date";
			cell.detailTextLabel.text = [dateFormat stringFromDate:[latestRecord date]];
			break;
		case 1:
			cell.textLabel.text = @"Buy";
			cell.detailTextLabel.text = [NSString stringWithFormat:@"%1.1f", [latestRecord.buy floatValue]];
			break;
		case 2:
			cell.textLabel.text = @"Sell";
			cell.detailTextLabel.text = [NSString stringWithFormat:@"%1.1f", [latestRecord.sell floatValue]];
			break;
		default:
			cell.textLabel.text = selectedCurrency.identifier;
			break;
	}
    
    return cell;
}


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    // Navigation logic may go here. Create and push another view controller.
	/*
	 <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
	 [self.navigationController pushViewController:detailViewController animated:YES];
	 [detailViewController release];
	 */
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc
{
	[plotViewController release];
	[dateFormat release];
    [super dealloc];
}


@end

