//
//  PlotViewController.m
//  Currency
//
//  Created by Hjalti Jakobsson on 17.8.2010.
//  Copyright 2010 Hjalti Jakobsson. All rights reserved.
//

#import "PlotViewController.h"
#import "Currency.h"
#import "Record.h"
#import <QuartzCore/QuartzCore.h>

#define kUseExampleData 1

@implementation PlotViewController
@synthesize currentCurrency;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
	if(self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])
	{
		self.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
		
		return self;
	}
	
	return nil;
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];
	
	[[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleBlackOpaque animated:NO];
	
	DPLineChartView *chartView = (DPLineChartView*)self.view;
	[chartView setLineColor:[UIColor colorWithRed:90.0/255.0 green:162.0/255.0 blue:80.0/255.0 alpha:1.0]];
	[chartView setDelegate:self];
	[chartView reloadData]; 
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
	if(UIInterfaceOrientationIsLandscape(toInterfaceOrientation))
	{
		DPLineChartView *chartView = (DPLineChartView*)self.view;
		[chartView setNeedsDisplay];
	}
}

- (void)viewWillAppear:(BOOL)animated
{
	[[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleBlackOpaque animated:NO];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleDefault];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
	return UIInterfaceOrientationIsLandscape(interfaceOrientation);
}

#pragma mark -
#pragma mark DPLineChartView Datasource

- (NSString*)headerForLineChartView:(DPLineChartView*)chartView
{
	return [currentCurrency identifier];
}

- (NSInteger)numberOfPointsInLineChartView:(DPLineChartView*)chartView
{
	if(kUseExampleData)
		return 10;
	
	return [[currentCurrency records] count];
}

- (float)lineChartView:(DPLineChartView*)chartView pointValueAtIndex:(NSInteger)index
{
	if(kUseExampleData)
	{	
		switch(index)
		{
			case 0:
				return 118.4;
				break;
			case 1:
				return 118.2;
				break;
			case 2:
				return 117.0;
				break;
			case 3:
				return 117.5;
				break;
			case 4:
				return 118.5;
				break;
			case 5:
				return 119.0;
				break;
			case 6:
				return 118.9;
				break;
			case 7:
				return 118.9;
				break;
			case 8:
				return 119.4;
				break;
			case 9:
				return 119.6;
				break;
		}
	}
	
	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"date" ascending:YES];	
	NSArray *allRecords = [[currentCurrency.records allObjects] sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	
	if([allRecords count] > index)
		return [[[allRecords objectAtIndex:index] buy] floatValue];
	
	return 0;
}

- (void)dealloc {
    [super dealloc];
}


@end
